{% extends 'wildewidgets/block.html' %}
{% load wildewidgets %}


{% block block_content %}
{% for item in entries %}
  <a id="{{ item.title|slugify }}"><div class="{{block_name}}__entry__title {{entry_title_css_class}} mb-4 h2 {% if not forloop.first %}mt-5{% endif %}">{{item.title}}</div></a>
  <div class="{{block_name}}__entry {{entry_css_class}}">
    {% wildewidgets item.widget %}
  </div>
{% endfor %}
{% endblock %}

