{% extends 'wildewidgets/block.html' %}
{% load wildewidgets %}

{% block block_content %}
<div class="{{block_name}}__title font-weight-bold mb-4 h2">{{ title }}</div>
<div class="{{block_name}}__list sticky-top">
{% if actions %}
    {% wildewidgets actions %}
{% endif %}
{% if widget_index %}
    {% wildewidgets widget_index %}
{% endif %}
</div>
{% endblock %}

