<div class="modal fade" id="{{modal_id}}" tabindex="-1" role="dialog" aria-labelledby="{{modal_id}}Label" aria-hidden="true">
  <div class="modal-dialog">
    <div class="modal-content pb-5">
      <div class="modal-header">
        <div class='h4' class="modal-title" id="{{modal_id}}Label">{{modal_title}}</div>
        <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
      </div>
      <div class="modal-body">
        {% block modal_body %}
        {% endblock %}
      </div>
    </div>
  </div>
</div>
